@@ -51,29 +51,29 @@ markdown <- function(x, info, ...) {
 
 #' @export
 print.lints <- function(x, ...) {
-  has_lints <- length(x) > 0
+  if (length(x)) {
+    if (getOption("lintr.rstudio_source_markers", TRUE) &&
+        rstudioapi::hasFun("sourceMarkers")) {
+      rstudio_source_markers(x)
+    } else {
+      if (in_ci() && settings$comment_bot) {
 
-  if (getOption("lintr.rstudio_source_markers", TRUE) &&
-      rstudioapi::hasFun("sourceMarkers")) {
-    rstudio_source_markers(x)
-  } else {
-    if (has_lints && in_ci() && settings$comment_bot) {
+        info <- ci_build_info()
 
-      info <- ci_build_info()
+        lint_output <-
+          trim_output(paste0(collapse = "\n",
+                             capture.output(invisible(lapply(x, markdown, info, ...)))
+                             )
+        )
 
-      lint_output <-
-        trim_output(paste0(collapse = "\n",
-                           capture.output(invisible(lapply(x, markdown, info, ...)))
-                           )
-      )
-
-      github_comment(lint_output, info, ...)
+        github_comment(lint_output, info, ...)
+      }
+      lapply(x, print, ...)
     }
-    lapply(x, print, ...)
-  }
 
-  if (has_lints && isTRUE(settings$error_on_lint)) {
-    quit("no", 31, FALSE)
+    if (isTRUE(settings$error_on_lint)) {
+      quit("no", 31, FALSE)
+    }
   }
   invisible(x)
 }
@@ -92,8 +92,11 @@ trim_output <- function(x, max = 65535) {
                  "(", except_some_of(")"), ")",
                  space,
                  "*", or("style", "warning", "error"), ":", "*",
-                 except_some_of(newline), newline, except_some_of(newline),
-                 newline, except_some_of(newline), newline)
+                 except_some_of("\r\n"), newline,
+                 except_some_of("\r\n"), newline,
+                 except_some_of("\r\n"), newline,
+                 except_some_of("\r\n"), newline,
+                 except_some_of("\r\n"), newline)
 
   lint_starts <- rex::re_matches(x, re, global = TRUE, locations = TRUE)[[1]]
 
@@ -141,3 +144,16 @@ as.data.frame.lints <- function(x, row.names = NULL, optional = FALSE, ...) {
   attributes(x) <- attrs
   x
 }
+
+#' @export
+summary.lints <- function(object, ...) {
+  filenames <- vapply(object, `[[`, character(1), "filename")
+  types <- factor(vapply(object, `[[`, character(1), "type"),
+    levels = c("style", "warning", "error"))
+  tbl <- table(filenames, types)
+  filenames <- rownames(tbl)
+  res <- as.data.frame.matrix(tbl, stringsAsFactors = FALSE, row.names = NULL)
+  res$filenames <- filenames %||% character()
+  nms <- colnames(res)
+  res[order(res$filenames), c("filenames", nms[nms != "filenames"])]
+}
