@@ -1,4 +1,5 @@
-context("open_curly_linter()")
+context("open_curly_linter")
+
 test_that("returns the correct linting", {
 
   expect_lint("blah", NULL, open_curly_linter())
@@ -37,4 +38,14 @@ test_that("returns the correct linting", {
   expect_lint("a <- function() { 1 }",
     NULL,
     open_curly_linter(allow_single_line = TRUE))
+
+  expect_lint(
+'if ("P" != "NP") { # what most people expect
+    print("Cryptomania is possible")
+}',
+    NULL,
+    open_curly_linter()
+  )
+
+  expect_lint('{{x}}', NULL, open_curly_linter())
 })
