@@ -17,6 +17,8 @@ test_that("returns the correct linting", {
 
   expect_lint("(1 + 1)", NULL, spaces_left_parentheses_linter)
 
+  expect_lint("(1 + 1)", NULL, spaces_left_parentheses_linter)
+
   expect_lint("( (1 + 1) )", NULL, spaces_left_parentheses_linter)
 
   expect_lint("if (blah) { }", NULL, spaces_left_parentheses_linter)
@@ -39,10 +41,6 @@ test_that("returns the correct linting", {
 
   expect_lint("c(function(){})[1]()", NULL, spaces_left_parentheses_linter)
 
-  expect_lint("((1 + 1))",
-    rex("Place a space before left parenthesis, except in a function call."),
-    spaces_left_parentheses_linter)
-
   expect_lint("if(blah) { }",
     rex("Place a space before left parenthesis, except in a function call."),
     spaces_left_parentheses_linter)
@@ -66,4 +64,12 @@ test_that("returns the correct linting", {
   expect_lint("\"test <- function(x) { if(1 + 1) 'hi' }\"",
     NULL,
     spaces_left_parentheses_linter)
+
+  expect_lint("res <- c((mat - 1L) %*% combs + 1L)",
+    NULL,
+    spaces_left_parentheses_linter)
+
+  expect_lint("if (!(foo && bar || baz)) { foo }",
+    NULL,
+    spaces_left_parentheses_linter)
 })
