@@ -5,8 +5,7 @@ test_that("returns the correct linting", {
     trailing_whitespace_linter)
 
   expect_lint("blah <- 1  ",
-    c(message = rex("Trailing whitespace is superfluous."),
-      column_number = 10),
+    list(message = rex("Trailing whitespace is superfluous."), column_number = 10),
     trailing_whitespace_linter)
 
   expect_lint("blah <- 1  \n'hi'",
@@ -14,11 +13,6 @@ test_that("returns the correct linting", {
     trailing_whitespace_linter)
 
   expect_lint("blah <- 1\n'hi'\na <- 2  ",
-    list(
-      c(
-        message = rex("Trailing whitespace is superfluous."),
-        line_number = 3
-      )
-    ),
+    list(message = rex("Trailing whitespace is superfluous."), line_number = 3),
     trailing_whitespace_linter)
 })
